from hamcrest.core import *
from hamcrest.library import *
from hamcrest import core, library
from hamcrest._version import version

__version__ = version
__author__ = "Chris Rose"
__copyright__ = "Copyright 2020 hamcrest.org"
__license__ = "BSD, see License.txt"

__all__ = []
__all__.extend(core.__all__)
__all__.extend(library.__all__)
